@@ -23,19 +23,26 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <QDebug>
+#include "textprovider.h"
 
 #define CLAMP(min, x, max) qMax(qMin(x, max), min)
 
 DataProvider::DataProvider(const QString& dir, QObject *parent) :
-  QObject(parent), m_dir(dir), m_index(-1), m_data(0) {
+  QObject(parent), m_dir(dir), m_index(-1), m_data(0), m_secondary(0) {
+
 }
 
 DataProvider::~DataProvider() {
   if (m_data) {
-    munmap(m_data, Texts[m_index].len);
+    delete m_data;
     m_data = 0;
     m_index = -1;
   }
+
+  if (m_secondary) {
+    delete m_secondary;
+    m_secondary = 0;
+  }
 }
 
 int DataProvider::pageCount() {
@@ -104,13 +111,6 @@ QString DataProvider::partName(int page) {
   return QString::fromUtf8(Parts[p->part].name);
 }
 
-// TODO:
-//int DataProvider::numberOfSuraPages(int sura) {
-//  return Suras[sura].length;
-//}
-
-
-
 bool DataProvider::hasPage(int page) const {
   return page >= MIN_PAGE && page <= MAX_PAGE;
 }
@@ -134,11 +134,7 @@ QString DataProvider::text(int sura, int aya) const {
 
   off_t index = Offsets[sura];
 
-  Aya *a = Texts[m_index].table + index + aya;
-
-  char *data = m_data + a->pos;
-
-  return QString::fromUtf8(data, a->len);
+  return m_data->text(aya, index);
 }
 
 QStringList DataProvider::text(const Fragment& frag) const {
@@ -155,6 +151,30 @@ QStringList DataProvider::text(const Fragment& frag) const {
   return ret;
 }
 
+QStringList DataProvider::secondaryText(const Fragment& frag) const {
+  if (!m_secondary) {
+    return QStringList();
+  }
+
+  QStringList ret;
+
+  for (int x = frag.start(); x < frag.size() + frag.start(); x++) {
+    off_t index = Offsets[frag.sura()];
+    ret << m_secondary->text(x, index);
+  }
+
+  return ret;
+}
+
+QString DataProvider::secondaryText(int sura, int aya) {
+  if (!m_secondary) {
+    return QString();
+  }
+
+  off_t index = Offsets[sura];
+  return m_secondary->text(aya, index);
+}
+
 QStringList DataProvider::availableTexts() const {
   QStringList texts;
 
@@ -170,51 +190,28 @@ int DataProvider::textType() const {
 }
 
 bool DataProvider::setTextType(int index) {
-  int idx = CLAMP(MIN_TEXT, index, MAX_TEXT);
-  if (idx != index) {
+  if (index != CLAMP(MIN_TEXT, index, MAX_TEXT)) {
     return false;
   }
 
-  if (idx == m_index) {
+  if (index == m_index) {
     return true;
   }
 
-  QString file = QString("%1%2%3").arg(m_dir).arg(QDir::separator()).arg(Texts[idx].name);
+  QString data = QString("%1%2%3").arg(m_dir).arg(QDir::separator()).arg(Texts[index].name);
+  QString idx = Texts[index].idx;
 
-  int fd = open(file.toLocal8Bit().data(), O_RDONLY);
-  if (fd == -1) {
-    return false;
-  }
-
-  struct stat buf;
-  if (fstat(fd, &buf) != 0) {
-    close(fd);
-    return false;
-  }
-
-  if (buf.st_size != Texts[idx].len) {
-    close(fd);
-    return false;
-  }
-
-  char *data = (char *)mmap(0, buf.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
-  if (data == MAP_FAILED) {
-    close(fd);
+  TextProvider *p = new TextProvider(index, data, idx);
+  if (!p->load()) {
+    delete p;
     return false;
   }
 
   if (m_data) {
-    if (munmap(m_data, Texts[m_index].len) != 0) {
-      close(fd);
-      munmap(data, buf.st_size);
-      return false;
-    }
+    delete m_data;
   }
 
-  m_data = data;
-  m_index = idx;
-
-  close(fd);
+  m_data = p;
 
   return true;
 }
@@ -273,6 +270,17 @@ QList<Fragment> Page::fragments() {
   return frags;
 }
 
+TextProvider *DataProvider::secondaryTextProvider() const {
+  return m_secondary;
+}
+
+void DataProvider::setSecondaryText(TextProvider *text) {
+  if (m_secondary != text) {
+    delete m_secondary;
+    m_secondary = text;
+  }
+}
+
 int Fragment::sura() const {
   return Fragments[m_index].sura;
 }
