@@ -25,6 +25,7 @@
 class Page;
 class Fragment;
 class Sura;
+class TextProvider;
 
 class DataProvider : public QObject {
   Q_OBJECT
@@ -44,9 +45,6 @@ public:
 
   Q_INVOKABLE int firstSuraForPage(int page);
 
-  // TODO:
-  //  Q_INVOKABLE int numberOfSuraPages(int sura);
-
   Q_INVOKABLE bool hasPage(int page) const;
 
   QStringList availableTexts() const;
@@ -68,12 +66,19 @@ public:
   Q_INVOKABLE QString text(int sura, int aya) const;
 
   QStringList text(const Fragment& frag) const;
+  QStringList secondaryText(const Fragment& frag) const;
+  QString secondaryText(int sura, int aya);
+
+  void setSecondaryText(TextProvider *text);
+
+  TextProvider *secondaryTextProvider() const;
 
 private:
   const QString m_dir;
 
   int m_index;
-  char *m_data;
+  TextProvider *m_data;
+  TextProvider *m_secondary;
 };
 
 class Page {
