@@ -25,11 +25,11 @@ Legal::~Legal() {
 
 }
 
-QString Legal::copyright() {
+QString Legal::copyright() const {
   return tr("Copyright &copy; 2011 <a href=\"mailto:msameer@foolab.org\">Mohammed Sameer</a>. All rights reserved.");
 }
 
-QString Legal::appLicense() {
+QString Legal::appLicense() const {
   return tr("This program is free software; you can redistribute it and/or modify "
 	    "it under the terms of the GNU General Public License as published by "
 	    "the Free Software Foundation; either version 2 of the License, or "
@@ -44,7 +44,7 @@ QString Legal::appLicense() {
 	    "along with this program. If not, see <a href=\"http://www.gnu.org/licenses/\">http://www.gnu.org/licenses</a>.");
 }
 
-QString Legal::textLicense() {
+QString Legal::textLicense() const {
   return tr("Tanzil Quran Text<br />"
 	    "Copyright &copy; 2008-2011 Tanzil Project<br />"
 	    "License: Creative Commons Attribution 3.0<br />"
@@ -69,3 +69,8 @@ QString Legal::textLicense() {
 	    "<br />"
 	    "Please check updates at: <a href=\"http://tanzil.net/updates/\">http://tanzil.net/updates</a>");
 }
+
+QString Legal::transLicense() const {
+  return tr("The translations provided are for non-commercial purposes only. If used otherwise, you need to obtain necessary permission from the translator or the publisher.<br />"
+	    "Translations are being downloaded from <a href=\"http://tanzil.net/trans/\">http://tanzil.net/trans/</a>");
+}
